/**
* @file main.cc
* @brief This is the main file of our project.
*
* @author Programación-II - GIR
*
* @date 2023/04/19
*/

/**
 * \mainpage Main page for the project.
 *
 * \section intro_sec Introduction
 *
 * This is the introduction.
 *
 * \section install_sec Installation
 *
 * \subsection step1 Step 1: Deployment strategies
 *
 * etc...
 */

#include <cstdint>
#include <cstdlib>
#include <iostream>
#include <string>
#include <vector>

/// Access std namespace members
using namespace std;

//! This class represents a Robot in out design
//! We construct a Robot from its name.
class Robot
{
public:
  Robot(const string& name); //!< Construtor from the name.
  ~Robot();                  //!< Destructor for the class. Prints Robot's name.
private:
  /** The name of our Robot */
  string name;
};

/// Type alias to clarify Robot pointer variables.
using RobotPtr = Robot*;

/**
 * @param name The name of the Robot being constructed.
 */
Robot::Robot(const string& name) { this->name = name; }
Robot::~Robot() { cout << "Destroying " << this->name << '\n'; }

/**
 * Creates a vector of RobotPtr.
 * @param nr The numbre of Robots in the vector.
 */
vector<RobotPtr> create_robots(uint32_t nr) {
  vector<RobotPtr> vrptr;
  vrptr.resize(nr);

  for (auto r = 0u; r < nr; r++)
    vrptr.push_back(new Robot("Robot-"+to_string(r)));

  return vrptr;
}

/// Frees the memory of the vector and the Robots inside it.
/// @param v The vector holding RobotPtr
void free_memory(vector<RobotPtr>& v) {
  for (auto rptr: v) delete rptr;
  v.clear();
}

/*!
 * Main function.
 *
 * Create the Robots and then free its memory.
 */
int main(int argc, char *argv[])
{
  uint32_t nr = 3;

  if (argc == 2)
    nr = atoi(argv[1]);

  cout << "Creating " << nr << " robots.\n";
  auto v = create_robots(nr);

  free_memory(v);

  return 0;
}
